import os
import pytest

from oneenv import (
    oneenv,
    template,
    generate_env_example,
    diff,
    set_key,
    unset_key,
    dotenv_values,
    load_dotenv,
    OneEnv,
    collect_templates,
    report_duplicates,
)
import oneenv as oneenv_module  # Used to clear the global registry

# Fixture to clear the template registry before each test
@pytest.fixture(autouse=True)
def clear_registry():
    oneenv_module._TEMPLATE_REGISTRY.clear()


# Define a sample template function for testing
@oneenv
def sample_template():
    """
    Returns a dictionary of environment variable settings for testing.
    """
    return {
        "TEST_VAR": {
            "description": "Test variable description.",
            "default": "value",
            "required": True,
            "choices": ["value", "other"]
        }
    }


def test_template_output():
    """
    Test that the generated template output contains expected lines:
    - The description
    - 'Required' if required is True
    - 'Choices: ...' if choices are provided
    - The variable assignment in the format DEFAULT=...
    """
    output = template()
    # Check for header and defined source
    assert "# Auto-generated by OneEnv" in output
    assert "# (Defined in: sample_template)" in output
    # Check description lines
    assert "# Test variable description." in output
    # Check for required and choices comments
    assert "# Required" in output
    assert "# Choices: value, other" in output
    # Check variable assignment
    assert "TEST_VAR=value" in output


def test_diff():
    """
    Test that diff() properly formats differences between texts.
    """
    previous_text = "TEST_VAR=old_value\n"
    current_text = "TEST_VAR=new_value\n"
    diff_output = diff(previous_text, current_text)
    # Expect a modification line in the format "~ old_line → new_line"
    expected_line = "~ TEST_VAR=old_value → TEST_VAR=new_value"
    assert expected_line in diff_output


def test_generate_env_example(tmp_path):
    """
    Test that generate_env_example() writes the same content as returned by template().
    """
    env_file = tmp_path / ".env.example"
    generate_env_example(str(env_file))
    written_content = env_file.read_text(encoding="utf-8")
    expected = template()
    assert written_content == expected


def test_set_and_unset_key(tmp_path):
    """
    Test set_key() and unset_key() functions by creating a temporary .env file,
    setting a key, updating it, then unsetting it.
    """
    env_file = tmp_path / ".env"
    # Start with an empty .env file
    env_file.write_text("", encoding="utf-8")
    
    # Set a key in the .env file and check content
    set_key(str(env_file), "MY_KEY", "123")
    content_after_set = env_file.read_text(encoding="utf-8")
    assert "MY_KEY=123" in content_after_set
    
    # Update the key value and check new content
    set_key(str(env_file), "MY_KEY", "456")
    content_after_update = env_file.read_text(encoding="utf-8")
    assert "MY_KEY=456" in content_after_update
    
    # Unset the key and verify it is removed
    unset_key(str(env_file), "MY_KEY")
    content_after_unset = env_file.read_text(encoding="utf-8")
    assert "MY_KEY=" not in content_after_unset


def test_dotenv_values(tmp_path):
    """
    Test that dotenv_values() correctly reads environment variables from a .env file.
    """
    env_file = tmp_path / ".env"
    env_file.write_text("MY_KEY=789\n", encoding="utf-8")
    values = dotenv_values(str(env_file))
    assert values["MY_KEY"] == "789"


def test_load_dotenv(tmp_path):
    """
    Test that load_dotenv() loads environment variables from a .env file into os.environ.
    """
    env_file = tmp_path / ".env"
    env_file.write_text("MY_LOAD_KEY=loaded_value\n", encoding="utf-8")
    # Ensure the environment variable is not already set
    if "MY_LOAD_KEY" in os.environ:
        del os.environ["MY_LOAD_KEY"]
    result = load_dotenv(str(env_file))
    assert result is True
    assert os.environ.get("MY_LOAD_KEY") == "loaded_value"


# English: Test class inheriting from OneEnv for testing purposes.
# Japanese: テスト用のOneEnvを継承したクラスです。
class TestLibTemplate(OneEnv):
    def get_template(self) -> dict:
        return {
            "TEST_LIB_KEY": {
                "description": "Test library key",
                "default": "test_value",
                "required": True
            }
        }


# English: Another test class for testing duplicate keys.
# Japanese: 重複キーのテスト用の別のクラスです。
class AnotherTestTemplate(OneEnv):
    def get_template(self) -> dict:
        return {
            "TEST_LIB_KEY": {  # Intentionally duplicate key
                "description": "Duplicate key for testing",
                "default": "another_value"
            }
        }


@oneenv
def test_template():
    """
    English: Test template function using the @oneenv decorator.
    Japanese: @oneenvデコレータを使用したテストテンプレート関数です。
    """
    return {
        "TEST_API_KEY": {
            "description": "Test API key",
            "default": "",
            "required": True
        }
    }


def test_oneenv_decorator():
    """
    English: Test that the @oneenv decorator correctly registers template functions.
    Japanese: @oneenvデコレータがテンプレート関数を正しく登録することをテストします。
    """
    templates = collect_templates()
    assert "TEST_API_KEY" in templates
    assert templates["TEST_API_KEY"]["config"]["description"] == "Test API key"
    assert templates["TEST_API_KEY"]["sources"] == ["test_template"]


def test_oneenv_subclass():
    """
    English: Test that OneEnv subclasses are correctly processed.
    Japanese: OneEnvサブクラスが正しく処理されることをテストします。
    """
    templates = collect_templates()
    assert "TEST_LIB_KEY" in templates
    assert templates["TEST_LIB_KEY"]["config"]["description"] == "Test library key"
    assert "TestLibTemplate" in templates["TEST_LIB_KEY"]["sources"]


def test_duplicate_keys():
    """
    English: Test that duplicate keys are properly handled and reported.
    Japanese: 重複キーが適切に処理され、報告されることをテストします。
    """
    templates = collect_templates()
    assert "TEST_LIB_KEY" in templates
    assert len(templates["TEST_LIB_KEY"]["sources"]) == 2
    assert "TestLibTemplate" in templates["TEST_LIB_KEY"]["sources"]
    assert "AnotherTestTemplate" in templates["TEST_LIB_KEY"]["sources"]


def test_template_generation():
    """
    English: Test that template generation includes both decorator and subclass templates.
    Japanese: テンプレート生成がデコレータとサブクラスの両方のテンプレートを含むことをテストします。
    """
    env_content = template()
    assert "TEST_API_KEY" in env_content
    assert "TEST_LIB_KEY" in env_content
    assert "Test API key" in env_content
    assert "Test library key" in env_content


def test_missing_description():
    """
    English: Test that templates without description raise ValueError.
    Japanese: 説明のないテンプレートがValueErrorを発生させることをテストします。
    """
    class InvalidTemplate(OneEnv):
        def get_template(self) -> dict:
            return {
                "INVALID_KEY": {
                    "default": "value"  # Missing description
                }
            }
    
    with pytest.raises(ValueError):
        collect_templates()


def test_required_field():
    """
    English: Test that required field is properly reflected in template output.
    Japanese: required フィールドがテンプレート出力に正しく反映されることをテストします。
    """
    env_content = template()
    assert "# Required" in env_content 